/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel;

import static org.apache.commons.lang.StringUtils.rightPad;
import static org.fortiss.tooling.kernel.internal.ToolingKernelInternal.initializeKernel;
import static org.fortiss.tooling.kernel.internal.ToolingKernelInternal.startKernel;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.IConnectionCompositor;
import org.fortiss.tooling.kernel.extension.IConstraintChecker;
import org.fortiss.tooling.kernel.extension.IConstraintVerifier;
import org.fortiss.tooling.kernel.extension.IElementCompositor;
import org.fortiss.tooling.kernel.extension.ILibraryElementHandler;
import org.fortiss.tooling.kernel.extension.IMigrationProvider;
import org.fortiss.tooling.kernel.extension.IPrototypeProvider;
import org.fortiss.tooling.kernel.extension.IStorageProvider;
import org.fortiss.tooling.kernel.extension.ITransformationProvider;
import org.fortiss.tooling.kernel.extension.LogMessageHandler;
import org.fortiss.tooling.kernel.extension.data.LogMessage;
import org.fortiss.tooling.kernel.internal.ToolingKernelInternal;
import org.fortiss.tooling.kernel.model.constraints.IConstraint;

/**
 * Class for centrally initializing and starting the tooling kernel services.
 * 
 * @author hoelzl
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating RED Hash:
 */
public final class ToolingKernel {
	/** Initialize the kernel. */
	public static void initialize() {
		initializeKernel();
	}

	/** Start the kernel. */
	public static void start() {
		startKernel();
	}

	/** Registers the given transformation with the service. */
	public static void registerTransformationProvider(ITransformationProvider provider,
			Class<?> sourceClass, Class<?> targetClass) {
		ToolingKernelInternal.registerTransformationProvider(provider, sourceClass, targetClass);
	}

	/** Registers the given provider with the kernel. */
	public static void registerPrototypeProvider(IPrototypeProvider provider) {
		ToolingKernelInternal.registerPrototypeProvider(provider);
	}

	/** Registers the given storage provider. */
	public static void registerModelStorageProvider(IStorageProvider provider) {
		ToolingKernelInternal.registerModelStorageProvider(provider);
	}

	/** Registers the migration provider with the service. */
	public static void registerMigrationProvider(IMigrationProvider provider,
			Class<?> modelElementClass) {
		ToolingKernelInternal.registerMigrationProvider(provider, modelElementClass);
	}

	/** Registers the given handler with the kernel. */
	public static void registerLibraryElementHandler(ILibraryElementHandler<EObject> handler,
			Class<?> modelElementClass) {
		ToolingKernelInternal.registerLibraryElementHandler(handler, modelElementClass);
	}

	/** Registers the given handler with the kernel. */
	public static <LM extends LogMessage> void registerLogMessageHandler(
			LogMessageHandler<LM> handler, Class<LM> logMessageClass) {
		ToolingKernelInternal.registerLogMessageHandler(handler, logMessageClass);
	}

	/** Registers the given compositor with the kernel. */
	public static void registerElementCompositor(IElementCompositor<EObject> compositor,
			Class<?> modelElementClass) {
		ToolingKernelInternal.registerElementCompositor(compositor, modelElementClass);
	}

	/** Registers the given verifier with the kernel. */
	public static <T extends IConstraint> void registerConstraintVerifier(
			IConstraintVerifier<T> verifier, Class<T> constraintElementClass) {
		ToolingKernelInternal.registerConstraintVerifier(verifier, constraintElementClass);
	}

	/** Registers the given constraint checker with the kernel. */
	public static void registerConstraintChecker(IConstraintChecker<EObject> checker,
			Class<?> modelElementClass) {
		ToolingKernelInternal.registerConstraintChecker(checker, modelElementClass);
	}

	/** Registers the given compositor with the kernel. */
	public static void registerConnectionCompositor(
			IConnectionCompositor<EObject, EObject, EObject> compositor, Class<?> sourceClass,
			Class<?> targetClass) {
		ToolingKernelInternal.registerConnectionCompositor(compositor, sourceClass, targetClass);
	}

	/** Prints the given kernel phase message to system console. */

	/** Prints the introduction message of the given phase */
	public static void printPhase(String phase) {
		phase = rightPad(phase, 40);
		System.out.println();
		System.out.println("********************************************");
		System.out.println("* " + phase + " *");
		System.out.println("********************************************");
		System.out.println();
	}
}
