/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import static org.fortiss.tooling.kernel.ToolingKernel.printPhase;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.IConnectionCompositor;
import org.fortiss.tooling.kernel.extension.IConstraintChecker;
import org.fortiss.tooling.kernel.extension.IConstraintVerifier;
import org.fortiss.tooling.kernel.extension.IElementCompositor;
import org.fortiss.tooling.kernel.extension.ILibraryElementHandler;
import org.fortiss.tooling.kernel.extension.IMigrationProvider;
import org.fortiss.tooling.kernel.extension.IPrototypeProvider;
import org.fortiss.tooling.kernel.extension.IStorageProvider;
import org.fortiss.tooling.kernel.extension.ITransformationProvider;
import org.fortiss.tooling.kernel.extension.LogMessageHandler;
import org.fortiss.tooling.kernel.extension.data.LogMessage;
import org.fortiss.tooling.kernel.internal.storage.eclipse.EclipseResourceStorageService;
import org.fortiss.tooling.kernel.model.constraints.IConstraint;
import org.fortiss.tooling.kernel.service.IConnectionCompositorService;
import org.fortiss.tooling.kernel.service.IConstraintCheckerService;
import org.fortiss.tooling.kernel.service.IConstraintVerificationService;
import org.fortiss.tooling.kernel.service.IEclipseResourceStorageService;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.service.ILibraryService;
import org.fortiss.tooling.kernel.service.ILoggingService;
import org.fortiss.tooling.kernel.service.IMigrationService;
import org.fortiss.tooling.kernel.service.IPersistencyService;
import org.fortiss.tooling.kernel.service.IPrototypeService;
import org.fortiss.tooling.kernel.service.ITransformationService;
import org.fortiss.tooling.kernel.service.ITutorialService;

/**
 * Implementation of the kernel initialization and startup.
 * 
 * @author hoelzl
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating RED Hash:
 */
public final class ToolingKernelInternal {
	/** Initialize the kernel services. */
	public static void initializeKernel() {
		printPhase("Tooling Kernel initializing ...");
		((ConnectionCompositorService)IConnectionCompositorService.getInstance())
				.initializeService();
		((ConstraintCheckerService)IConstraintCheckerService.getInstance()).initializeService();
		((ConstraintVerificationService)IConstraintVerificationService.getInstance())
				.initializeService();
		((ElementCompositorService)IElementCompositorService.getInstance()).initializeService();
		((EclipseResourceStorageService)IEclipseResourceStorageService.getInstance())
				.initializeService();
		((LibraryService)ILibraryService.getInstance()).initializeService();
		((LoggingService)ILoggingService.getInstance()).initializeService();
		((MigrationService)IMigrationService.getInstance()).initializeService();
		((PersistencyService)IPersistencyService.getInstance()).initializeService();
		((PrototypeService)IPrototypeService.getInstance()).initializeService();
		((TransformationService)ITransformationService.getInstance()).initializeService();
		((TutorialService)ITutorialService.getInstance()).initializeService();
		printPhase("Tooling Kernel initialized.");
	}

	/** Starts the kernel services. */
	public static void startKernel() {
		printPhase("Tooling Kernel starting ...");
		((ConnectionCompositorService)IConnectionCompositorService.getInstance()).startService();
		((ConstraintCheckerService)IConstraintCheckerService.getInstance()).startService();
		((ConstraintVerificationService)IConstraintVerificationService.getInstance())
				.startService();
		((ElementCompositorService)IElementCompositorService.getInstance()).startService();
		((EclipseResourceStorageService)IEclipseResourceStorageService.getInstance())
				.startService();
		((LibraryService)ILibraryService.getInstance()).startService();
		((LoggingService)ILoggingService.getInstance()).startService();
		((MigrationService)IMigrationService.getInstance()).startService();
		((PersistencyService)IPersistencyService.getInstance()).startService();
		((PrototypeService)IPrototypeService.getInstance()).startService();
		((TransformationService)ITransformationService.getInstance()).startService();
		((TutorialService)ITutorialService.getInstance()).startService();
		printPhase("Tooling Kernel started.");
	}

	/** Registers the given transformation with the kernel. */
	public static void registerTransformationProvider(ITransformationProvider provider,
			Class<?> sourceClass, Class<?> targetClass) {
		((TransformationService)ITransformationService.getInstance())
				.registerTransformationProvider(provider, sourceClass, targetClass);
	}

	/** Registers the given provider with the kernel. */
	public static void registerPrototypeProvider(IPrototypeProvider provider) {
		((PrototypeService)IPrototypeService.getInstance()).registerPrototypeProvider(provider);
	}

	/** Registers the given storage provider with the kernel. */
	public static void registerModelStorageProvider(IStorageProvider provider) {
		((PersistencyService)IPersistencyService.getInstance())
				.registerModelStorageProvider(provider);
	}

	/** Registers the migration provider with the service. */
	public static void registerMigrationProvider(IMigrationProvider provider,
			Class<?> modelElementClass) {
		((MigrationService)IMigrationService.getInstance()).registerMigrationProvider(provider,
				modelElementClass);
	}

	/** Registers the given handler with the kernel. */
	public static void registerLibraryElementHandler(ILibraryElementHandler<EObject> handler,
			Class<?> modelElementClass) {
		((LibraryService)ILibraryService.getInstance()).registerLibraryElementHandler(handler,
				modelElementClass);
	}

	/** Registers the given handler with the kernel. */
	public static <LM extends LogMessage> void registerLogMessageHandler(
			LogMessageHandler<LM> handler, Class<LM> logMessageClass) {
		((LoggingService)ILoggingService.getInstance()).registerLogMessageHandler(handler,
				logMessageClass);
	}

	/** Registers the given compositor with the kernel. */
	public static void registerElementCompositor(IElementCompositor<EObject> compositor,
			Class<?> modelElementClass) {
		((ElementCompositorService)IElementCompositorService.getInstance())
				.registerElementCompositor(compositor, modelElementClass);
	}

	/** Registers the given verifier with the kernel. */
	public static <T extends IConstraint> void registerConstraintVerifier(
			IConstraintVerifier<T> verifier, Class<T> constraintElementClass) {
		((ConstraintVerificationService)IConstraintVerificationService.getInstance())
				.registerConstraintVerifier(verifier, constraintElementClass);
	}

	/** Registers the given constraint checker with the kernel. */
	public static void registerConstraintChecker(IConstraintChecker<EObject> checker,
			Class<?> modelElementClass) {
		((ConstraintCheckerService)IConstraintCheckerService.getInstance())
				.registerConstraintChecker(checker, modelElementClass);
	}

	/** Registers the given compositor with the kernel. */
	public static void registerConnectionCompositor(
			IConnectionCompositor<EObject, EObject, EObject> compositor, Class<?> sourceClass,
			Class<?> targetClass) {
		((ConnectionCompositorService)IConnectionCompositorService.getInstance())
				.registerConnectionCompositor(compositor, sourceClass, targetClass);
	}
}
